﻿using System.Collections.Generic;
using System.Linq;
using DemoApp.Controller;
using DemoApp.CustomerModule.DataAccess;
using DemoApp.CustomerModule.Model;
using DemoApp.CustomerModule.ViewModel;
using DemoApp.CustomerModule.Properties;
using DemoApp.ViewModel;
using MvvmFoundation.Wpf;

namespace DemoApp.CustomerModule.Controller
{
    public class CustomerController : IController
    {
        #region Fields

        private readonly ICustomerRepository _customerRepository;
        private readonly IMainWindowViewModel _mainWindowViewModel;

        #endregion // Fields

        #region Constructor

        public CustomerController(IMainWindowViewModel mainWindowViewModel, ICustomerRepository customerRepository)
        {
            _mainWindowViewModel = mainWindowViewModel;
            _customerRepository = customerRepository;

            foreach (CommandViewModel commandViewModel in CreateCommands())
            {
                _mainWindowViewModel.Commands.Add(commandViewModel);
            }
        }

        #endregion // Constructor

        #region Private Helpers

        private IEnumerable<CommandViewModel> CreateCommands()
        {
            return new List<CommandViewModel>
                {
                    new CommandViewModel(
                        Strings.MainWindowViewModel_Command_ViewAllCustomers,
                        new RelayCommand(ShowAllCustomers)),
                    new CommandViewModel(
                        Strings.MainWindowViewModel_Command_CreateNewCustomer,
                        new RelayCommand(CreateNewCustomer))
                };
        }

        private void CreateNewCustomer()
        {
            Customer newCustomer = Customer.CreateNewCustomer();
            var workspace = new CustomerViewModel(newCustomer, _customerRepository);
            _mainWindowViewModel.Workspaces.Add(workspace);

            _mainWindowViewModel.SetActiveWorkspace(workspace);
        }

        private void ShowAllCustomers()
        {
            var workspace =
                _mainWindowViewModel.Workspaces.FirstOrDefault(vm => vm is AllCustomersViewModel)
                as AllCustomersViewModel;

            if (workspace == null)
            {
                workspace = new AllCustomersViewModel(_customerRepository);
                _mainWindowViewModel.Workspaces.Add(workspace);
            }

            _mainWindowViewModel.SetActiveWorkspace(workspace);
        }

        #endregion // Private Helpers
    }
}